"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InteractionResponseFlags = exports.InteractionResponseType = exports.InteractionType = exports.ApplicationCommandOptionType = exports.Interaction = void 0;
const axios_1 = require("axios");
/** Handle a slash command interaction */
class Interaction {
    constructor(interaction, p2, p3) {
        /** Whether or not an initial response been sent */
        this.sentInitial = false;
        this.applicationId = process.env.APPLICATION_ID;
        this.defaults = {
            type: InteractionResponseType.ChannelMessageWithSource,
        };
        if (p2) {
            if (typeof p2 === 'string') {
                this.applicationId = p2;
            }
            else {
                this.defaults = p2;
            }
        }
        if (p3) {
            this.defaults = p3;
        }
        Object.assign(this, interaction);
    }
    async send(body) {
        const url = this.sentInitial ? this.webhookURL : this.callbackURL;
        let data = {};
        if (this.sentInitial) {
            if (typeof body === 'string') {
                data = { content: body };
            }
            else {
                data = body;
            }
        }
        else {
            if (typeof body === 'string') {
                data = { ...this.defaults, data: { content: body } };
            }
            else {
                data = { ...this.defaults, ...body };
            }
        }
        const result = await axios_1.default.post(url, data, {
            validateStatus: null,
        });
        this.sentInitial = true;
        return result.data;
    }
    /**
     * Edit an interaction response
     * @param body The new message content
     * @param [id] The ID to of the message to edit. Leave blank to edit original message
     */
    async edit(body, id = '@original') {
        const result = await axios_1.default.patch(`${this.webhookURL}/messages/${id}`, typeof body === 'string' ? { content: body } : body, {
            validateStatus: null,
        });
        return result.data;
    }
    /**
     * Delete a message
     * @param [id] The ID of the message to delete. Leave empty to delete original message.
     */
    async delete(id = '@original') {
        const result = await axios_1.default.delete(`${this.webhookURL}/messages/${id}`, {
            validateStatus: null,
        });
        return result.data;
    }
    /** The full command that was used */
    toString() {
        const fullcmd = [this.data.name];
        const descend = (s) => {
            for (const op of s) {
                fullcmd.push((op === null || op === void 0 ? void 0 : op.value) ? `${op.name}:` : op.name);
                if (op.options) {
                    descend(op.options);
                }
                if (op.value !== undefined) {
                    fullcmd.push(op.value);
                }
            }
        };
        if (this.data.options) {
            descend(this.data.options);
        }
        return '/' + fullcmd.join(' ');
    }
    /** The callback URL for sending an initial response */
    get callbackURL() {
        return `https://discord.com/api/v8/interactions/${this.id}/${this.token}/callback`;
    }
    /** The webhook URL for sending followup messages */
    get webhookURL() {
        return `https://discord.com/api/v8/webhooks/${this.applicationId}/${this.token}`;
    }
    /** The full command that was used */
    get content() {
        return this.toString();
    }
}
exports.Interaction = Interaction;
var ApplicationCommandOptionType;
(function (ApplicationCommandOptionType) {
    ApplicationCommandOptionType[ApplicationCommandOptionType["SUB_COMMAND"] = 1] = "SUB_COMMAND";
    ApplicationCommandOptionType[ApplicationCommandOptionType["SUB_COMMAND_GROUP"] = 2] = "SUB_COMMAND_GROUP";
    ApplicationCommandOptionType[ApplicationCommandOptionType["STRING"] = 3] = "STRING";
    ApplicationCommandOptionType[ApplicationCommandOptionType["INTEGER"] = 4] = "INTEGER";
    ApplicationCommandOptionType[ApplicationCommandOptionType["BOOLEAN"] = 5] = "BOOLEAN";
    ApplicationCommandOptionType[ApplicationCommandOptionType["USER"] = 6] = "USER";
    ApplicationCommandOptionType[ApplicationCommandOptionType["CHANNEL"] = 7] = "CHANNEL";
    ApplicationCommandOptionType[ApplicationCommandOptionType["ROLE"] = 8] = "ROLE";
})(ApplicationCommandOptionType = exports.ApplicationCommandOptionType || (exports.ApplicationCommandOptionType = {}));
var InteractionType;
(function (InteractionType) {
    InteractionType[InteractionType["Ping"] = 1] = "Ping";
    InteractionType[InteractionType["ApplicationCommand"] = 2] = "ApplicationCommand";
})(InteractionType = exports.InteractionType || (exports.InteractionType = {}));
var InteractionResponseType;
(function (InteractionResponseType) {
    InteractionResponseType[InteractionResponseType["Pong"] = 1] = "Pong";
    InteractionResponseType[InteractionResponseType["Acknowledge"] = 2] = "Acknowledge";
    InteractionResponseType[InteractionResponseType["ChannelMessage"] = 3] = "ChannelMessage";
    InteractionResponseType[InteractionResponseType["ChannelMessageWithSource"] = 4] = "ChannelMessageWithSource";
    InteractionResponseType[InteractionResponseType["ACKWithSource"] = 5] = "ACKWithSource";
})(InteractionResponseType = exports.InteractionResponseType || (exports.InteractionResponseType = {}));
var InteractionResponseFlags;
(function (InteractionResponseFlags) {
    InteractionResponseFlags[InteractionResponseFlags["EPHEMERAL"] = 64] = "EPHEMERAL";
})(InteractionResponseFlags = exports.InteractionResponseFlags || (exports.InteractionResponseFlags = {}));
//# sourceMappingURL=Interaction.js.map